const bluebird = require('bluebird')
const path = require('path')

const MONGO_HOSTS = process.env.LEGO_MONGO_HOSTS || '127.0.0.1:27017'
const MONGO_DB = process.env.LEGO_MONGO_DB
const MONGO_USER = process.env.LEGO_MONGO_USER
const MONGO_PASSWORD = process.env.LEGO_MONGO_PASSWORD
const MONGO_REPLSET = process.env.LEGO_MONGO_REPLSET

if (!MONGO_DB) {
  throw new Error('LEGO_MONGO_DB is required')
}

const MongoClient = require('mongodb').MongoClient
const url = `mongodb://${MONGO_HOSTS}`

const mongoClientOptions = {
  useNewUrlParser: true
}
if (MONGO_USER) {
  mongoClientOptions.authSource = MONGO_DB || 'admin'
  mongoClientOptions.auth = {
    user: MONGO_USER,
    password: MONGO_PASSWORD
  }
}

if (MONGO_REPLSET) {
  mongoClientOptions.replicaSet = MONGO_REPLSET
}

const mongo = new MongoClient(url, mongoClientOptions)

bluebird.promisifyAll(mongo)

const run = async function() {
  try {
    const conn = await mongo.connect()
    const db = conn.db(MONGO_DB)
    const systemData = require(path.join(__dirname, 'system.json'))
    const bussinessData = require(path.join(__dirname, 'bussiness.json'))
    const datas = [systemData, bussinessData]
    for (let data of datas) {
      for (let classDef of data) {
        let className = classDef.className
        const collection = db.collection(className)
        bluebird.promisifyAll(collection)
        await collection.remove({ system: true })
        console.log('removed')
        for (let object of classDef.objects) {
          try {
            await collection.insertOne(object)
            console.log('inserted:', object._id)
          } catch (e) {}
        }
      }
    }
    process.exit(0)
  } catch (e) {
    console.log(e)
    process.exit(1)
  }
}
run()
