import Parse from 'parse'

export default {
  namespaced: true,
  state: {
    total: 0,
    page: 1,
    pagesize: 10,
    tableData: [],
    keys: [],
    langs: [],
    allTableData: []
  },
  mutations: {
    setTableData(state, data) {
      state.tableData = data
    },
    setAllTableData(state, data) {
      state.allTableData = data
    },
    setTotal(state, total) {
      state.total = total
    },
    setPage(state, page) {
      state.page = page
    },
    setPagesize(state, pagesize) {
      state.pagesize = pagesize
    },
    setAllKeys(state, keys) {
      state.keys = keys
    },
    setLangs(state, langs) {
      state.langs = langs
    }
  },
  actions: {
    initTableData({ commit, state }, data) {
      let tableData
      let tableRowMap = {}
      let langs
      let keyword = (data.keyword || '').toLowerCase()
      let q = new Parse.Query('LEGO_Lang')
      return q
        .find()
        .then(res => {
          langs = res
          commit('setLangs', langs)
          q = new Parse.Query('LEGO_Lang_Res')
          return q.addDescending('createdAt').distinct('key')
        })
        .then(keys => {
          commit('setAllKeys', keys)
          tableData = keys.map(key => {
            let row = {
              key,
              values: {}
            }
            for (let lang of langs) {
              row.values[lang.get('name')] = ''
            }
            tableRowMap[key] = row
            return row
          })
          q = new Parse.Query('LEGO_Lang_Res')
          return q
            .containedIn('key', keys)
            .limit(9999)
            .include(data.fields.map(field => field.field).join(','))
            .addDescending('createdAt')
            .find()
        })
        .then(res => {
          for (let item of res) {
            let key = item.get('key')
            let row = tableRowMap[key]
            let lang = item.get('lang').get('name')
            row.values[lang] = item.get('value')
          }
          tableData = tableData.filter(row => {
            if (keyword) {
              return (
                row.key.toLowerCase().indexOf(keyword) !== -1 ||
                Object.keys(row.values).some(lang => {
                  return row.values[lang].toLowerCase().indexOf(keyword) !== -1
                })
              )
            }
            return true
          })
          commit('setTotal', tableData.length)
          commit('setAllTableData', tableData)
          commit('setTableData', tableData.slice((state.page - 1) * state.pagesize, state.page * state.pagesize))
        })
    },
    getTableData({ commit, state }, data) {
      let tableData = state.allTableData
      commit('setTableData', tableData.slice((state.page - 1) * state.pagesize, state.page * state.pagesize))
    }
  }
}
