import Parse from 'parse'
const moment = require('moment')

const fetchTableData = function({ commit, state }) {
  let q1 = new Parse.Query(state.className)
  q1.equalTo('LEGO_STATUS', null)
  let q2 = new Parse.Query(state.className)
  q2.equalTo('LEGO_STATUS', '')
  let q3 = new Parse.Query(state.className)
  q3.equalTo('LEGO_STATUS', 'delete')
  let q = Parse.Query.or(q1, q2, q3)
  let contains = state.contains
  if (contains) {
    let query = new Parse.Query(state.className)
    Object.keys(contains).forEach(key => {
      query.contains(key, contains[key])
    })
    q = Parse.Query.and(q, query)
  }
  let where = state.where
  if (where) {
    where = Parse.Query.fromJSON(state.className, { where })
    q = Parse.Query.and(q, where)
  }
  if (state.local) {
    commit('setLocal', state.local)
    if (state.tableData.length === 0) {
      q.limit(999)
    } else {
      let start = state.pagesize * (state.page - 1)
      commit('setTableData', state.rawData.slice(start, start + state.pagesize))
      return
    }
  } else {
    q.limit(state.pagesize).skip(state.pagesize * (state.page - 1))
  }
  return q
    .include(state.fields.map(field => field.field).join(','))
    .addDescending('createdAt')
    .find()
    .then(res => {
      commit('setRawData', res.map(item => item.toJSON()))
      if (state.local) {
        let start = state.pagesize * (state.page - 1)
        commit('setTableData', res.map(item => item.toJSON()).slice(start, start + state.pagesize))
      } else {
        commit('setTableData', res.map(item => item.toJSON()))
      }
      return q.count()
    })
    .then(count => {
      commit('setTotal', count)
    })
}

export default {
  namespaced: true,
  state: {
    rawData: [],
    total: 0,
    page: 1,
    pagesize: 10,
    tableData: [],
    local: false,
    className: '',
    fields: [],
    where: null,
    contains: null
  },
  mutations: {
    setRawData(state, data) {
      state.rawData = data
    },
    setTableData(state, data) {
      state.tableData = data
    },
    setTotal(state, total) {
      state.total = total
    },
    setPage(state, page) {
      state.page = page
    },
    setPagesize(state, pagesize) {
      state.pagesize = pagesize
    },
    setLocal(state, local) {
      state.local = local
    },
    set(state, { key, value }) {
      state[key] = value
    }
  },
  actions: {
    initTableData({ commit, state }, data) {
      commit('set', { key: 'contains', value: data.contains })
      commit('set', { key: 'where', value: data.where })
      commit('set', { key: 'className', value: data.className })
      commit('set', { key: 'fields', value: data.fields })
      return fetchTableData({ commit, state })
    },
    turn({ commit, state }, page) {
      commit('setPage', page)
      if (state.local) {
        let start = state.pagesize * (state.page - 1)
        commit('setTableData', state.rawData.slice(start, start + state.pagesize))
      } else {
        return fetchTableData({ commit, state })
      }
    },
    search({ commit, state }, { contains, local, fields }) {
      const i18n = window.i18n
      if (local) {
        let filteredData = state.rawData.filter(row => {
          return Object.keys(contains).some(key => {
            let field = fields.filter(field => field.field === key)[0]
            let value = row[key]
            if (field.templateSwitch) {
              let functionBody = field.template
                .trim()
                .replace(/[^\{]+\{/, '')
                .replace(/\}$/, '')
              let fn = new Function('row', 'moment', functionBody)
              value = fn(row, moment)
            }
            return (
              i18n
                .t(value)
                .toLowerCase()
                .indexOf(contains[key].toLowerCase()) !== -1
            )
          })
        })
        let start = state.pagesize * (state.page - 1)
        commit('setTotal', filteredData.length)
        commit('setTableData', filteredData.slice(start, start + state.pagesize))
      } else {
        commit('set', { key: 'contains', value: contains })
        return fetchTableData({ commit, state })
      }
    }
  }
}
