/*
 * 根据页面的配置，实例化vue对象
 */

import Vue from 'vue'
import VueRouter from 'vue-router'
import { configRouter } from 'utils/configRouter'
import RoutedApp from 'components/routed-app'
import ElementUI from 'element-ui'
import VueI18n from 'vue-i18n'

Vue.use(VueI18n)

/*
 * @route 页面的路由配置
 * @store 页面的数据仓库
 * @el 根节点选择器
 */
module.exports = function(opts) {
  const { route, store, el, component, i18n } = opts
  Vue.use(ElementUI, {
    i18n: (key, value) => i18n.t(key, value)
  })
  // 如果定义了路由配置，则用routed-app作为根组件
  if (route) {
    Vue.use(VueRouter)
    const transition = route.transition === undefined ? true : !!route.transition
    configRouter(route).then(router => {
      const app = new Vue({ template: `<App :animate=${transition}></App>`, components: { App: RoutedApp }, router, store, i18n })
      app.$mount(el || 'app')
    })
  } else {
    // 否则使用app作为根组件
    const app = new Vue({ template: '<App/>', components: { App: component }, store, i18n })
    app.$mount(el || 'app')
  }
}
