'use strict';

const assert = require('assert');
const r2 = require('r2');
// const urlBase = 'http://itougu.jrj.com.cn/act/crud'
const urlBase = 'http://localhost:3000/crud';

const createDoc = function(data) {
  data.__test = true;
  return data;
};

describe('crud', function() {
  this.timeout(10 * 1000);

  describe('clean', function() {
    it('delete all', async function() {
      await r2({
        headers: {
          'content-type': 'application/json'
        },
        url: `${urlBase}/test?where=${JSON.stringify({ __test: true })}`,
        method: 'delete'
      });
      let result = await r2({
        headers: {
          'content-type': 'application/json'
        },
        url: `${urlBase}/test?where=${JSON.stringify({ __test: true })}`
      }).json;
      assert.equal(0, result.length);
    });
  });

  describe('post', function() {
    it('create a new doc', async function() {
      let result = await r2({
        headers: {
          'content-type': 'application/json'
        },
        url: `${urlBase}/test`,
        method: 'post',
        json: createDoc({
          name: 'a1'
        })
      }).json;
      assert.equal('a1', result.name);
    });

    it('create some doc', async function() {
      let result = await r2({
        headers: {
          'content-type': 'application/json'
        },
        url: `${urlBase}/test`,
        method: 'post',
        json: createDoc({
          name: 'a3'
        })
      }).json;
      assert.equal('a3', result.name);
      result = await r2({
        headers: {
          'content-type': 'application/json'
        },
        url: `${urlBase}/test`,
        method: 'post',
        json: createDoc({
          name: 'a5'
        })
      }).json;
      assert.equal('a5', result.name);
      result = await r2({
        headers: {
          'content-type': 'application/json'
        },
        url: `${urlBase}/test`,
        method: 'post',
        json: createDoc({
          name: 'a2'
        })
      }).json;
      assert.equal('a2', result.name);
      result = await r2({
        headers: {
          'content-type': 'application/json'
        },
        url: `${urlBase}/test`,
        method: 'post',
        json: createDoc({
          name: 'a4'
        })
      }).json;
      assert.equal('a4', result.name);
    });
  });

  describe('get', async function() {
    it('get all docs', async function() {
      let result = await r2({
        headers: {
          'content-type': 'application/json'
        },
        url: `${urlBase}/test?where=${JSON.stringify({ __test: true })}`
      }).json;
      assert.equal(5, result.length);
    });

    it('limit', async function() {
      let result = await r2({
        headers: {
          'content-type': 'application/json'
        },
        url: `${urlBase}/test?where=${JSON.stringify({ __test: true })}&limit=2`
      }).json;
      assert.equal(2, result.length);
    });

    it('skip', async function() {
      let result = await r2({
        headers: {
          'content-type': 'application/json'
        },
        url: `${urlBase}/test?where=${JSON.stringify({
          __test: true
        })}&limit=2&skip=2`
      }).json;
      assert.equal(2, result.length);
      assert.equal('a5', result[0].name);
    });

    it('order asc', async function() {
      let result = await r2({
        headers: {
          'content-type': 'application/json'
        },
        url: `${urlBase}/test?where=${JSON.stringify({
          __test: true
        })}&order=name`
      }).json;
      assert.equal('a5', result[4].name);
    });

    it('order desc', async function() {
      let result = await r2({
        headers: {
          'content-type': 'application/json'
        },
        url: `${urlBase}/test?where=${JSON.stringify({
          __test: true
        })}&order=-name`
      }).json;
      assert.equal('a5', result[0].name);
    });

    it('where', async function() {
      let result = await r2({
        headers: {
          'content-type': 'application/json'
        },
        url: `${urlBase}/test?where=${JSON.stringify({
          name: 'a3',
          __test: true
        })}`
      }).json;
      assert.equal('a3', result[0].name);
    });

    it('objectId', async function() {
      let result = await r2({
        headers: {
          'content-type': 'application/json'
        },
        url: `${urlBase}/test?where=${JSON.stringify({ name: 'a3' })}`
      }).json;
      result = await r2({
        headers: {
          'content-type': 'application/json'
        },
        url: `${urlBase}/test/${result[0]._id}`
      }).json;
      assert.equal('a3', result.name);
    });
  });

  describe('put', function() {
    it('update a3 to a6', async function() {
      let result = await r2({
        headers: {
          'content-type': 'application/json'
        },
        url: `${urlBase}/test?where=${JSON.stringify({
          name: 'a3'
        })}`
      }).json;
      let objectId = result[0]._id;
      result = await r2({
        headers: {
          'content-type': 'application/json'
        },
        url: `${urlBase}/test/${objectId}`,
        method: 'put',
        json: {
          name: 'a6'
        }
      }).json;
      assert.equal('a6', result.name);
      result = await r2({
        headers: {
          'content-type': 'application/json'
        },
        url: `${urlBase}/test/${objectId}`
      }).json;
      assert.equal('a6', result.name);
      assert.ok(result.updatedAt);
    });
  });

  describe('delete', function() {
    it('delete a5', async function() {
      let result = await r2({
        headers: {
          'content-type': 'application/json'
        },
        url: `${urlBase}/test?where=${JSON.stringify({ name: 'a5' })}`,
        qs: {
          where: JSON.stringify({
            name: 'a5'
          })
        }
      }).json;
      let objectId = result[0]._id;
      await r2({
        headers: {
          'content-type': 'application/json'
        },
        url: `${urlBase}/test/${objectId}`,
        method: 'delete'
      });
      result = await r2({
        headers: {
          'content-type': 'application/json'
        },
        url: `${urlBase}/test?where=${JSON.stringify({ name: 'a5' })}`
      }).json;
      assert.equal(result.length, 0);
    });
  });
});
